@@ -422,6 +422,15 @@ static void got_buddy_list_cb(FacebookAccount *fba, const gchar *data,
 	gchar *error = NULL;
 	JsonObject *objnode = fb_get_json_object(parser, &error);
 	if (error) {
+		purple_debug_info("facebook", "eion test\n");
+		if (json_node_get_int(json_object_get_member(objnode, "error")) == 1356007)
+		{
+			//Chat is down for maintenence
+			purple_connection_error_reason(fba->pc,
+				PURPLE_CONNECTION_ERROR_AUTHENTICATION_IMPOSSIBLE,
+				error);
+			fba->pc->wants_to_die = TRUE;
+		}
 		if (fba->bad_buddy_list_count++ == 3)
 		{
 			purple_connection_error_reason(
@@ -429,6 +438,7 @@ static void got_buddy_list_cb(FacebookAccount *fba, const gchar *data,
 				PURPLE_CONNECTION_ERROR_NETWORK_ERROR,
 				error);
 		}
+		g_free(error);
 		json_parser_free(parser);
 		
 		return;
@@ -522,7 +532,7 @@ gboolean fb_get_buddy_list(gpointer data)
 	postdata = g_strdup_printf(
 			"user=%" G_GINT64_FORMAT "&popped_out=true&force_render=true&buddy_list=1&__a=1&post_form_id_source=AsyncRequest&post_form_id=%s&fb_dtsg=%s&notifications=1",
 			fba->uid, fba->post_form_id?fba->post_form_id:"(null)", fba->dtsg?fba->dtsg:"(null)");
-	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/presence/update.php",
+	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/chat/buddy_list.php",
 			postdata, got_buddy_list_cb, NULL, FALSE);
 	g_free(postdata);
 	
@@ -582,13 +592,21 @@ static void got_full_buddy_list(FacebookAccount *fba, const gchar *data,
 	{
 		node = json_node_get_object(json_array_get_element(entries, i));
 		const gchar *type = json_node_get_string(json_object_get_member(node, "ty"));
-		if (type[0] != 'u')
+		if (type[0] != 'u' && type[0] != 'g')
 			continue;
 		const gchar *uid = json_node_get_string(json_object_get_member(node, "i"));
+		const gchar *name = json_node_get_string(json_object_get_member(node, "t"));
+		
+		if (type[0] == 'g')
+		{
+			PurpleChat *chat = purple_blist_find_chat(fba->account, uid);
+			purple_blist_alias_chat(chat, name);
+			continue;
+		}
+		
 		if (purple_find_buddy(fba->account, uid))
 			continue;
 		
-		const gchar *name = json_node_get_string(json_object_get_member(node, "t"));
 		PurpleBuddy *buddy = purple_buddy_new(fba->account, uid, name);
 		purple_blist_add_buddy(buddy, NULL, fb_group, NULL);
 		FacebookBuddy *fbuddy = g_new0(FacebookBuddy, 1);
