@@ -34,5 +34,8 @@ void fb_history_fetch(FacebookAccount *fba, const char *who,
 void fb_conversation_handle_message(FacebookAccount *fba, const char *from,
 	const char *to, gint64 message_time, const gchar *message_orig,
 	gboolean log);
+void fb_conversation_handle_chat(FacebookAccount *fba, const char *from,
+	const char *group, gint64 message_time, const gchar *message_orig,
+	gboolean log);
 
 #endif /* FACEBOOK_CONVERSATION_H */
