@@ -23,6 +23,8 @@
 #include "fb_conversation.h"
 #include "fb_blist.h"
 #include "fb_util.h"
+#include "fb_chat.h"
+#include "fb_friendlist.h"
 
 #include "conversation.h"
 
@@ -71,7 +73,8 @@ void fb_cancel_resending_messages(FacebookAccount *fba)
 }
 
 static void parse_message(PurpleConnection *pc, FacebookAccount *fba,
-	JsonObject *messageObj, const char* from, const char* to)
+	JsonObject *messageObj, const char* from, const char* to,
+	PurpleConversationType type)
 {
 	gint64 message_time;
 	const gchar *message;
@@ -84,7 +87,10 @@ static void parse_message(PurpleConnection *pc, FacebookAccount *fba,
 	message_time = fb_time_kludge(json_node_get_int(
 				json_object_get_member(messageObj, "time")));
 
-	fb_conversation_handle_message(fba, from, to, message_time, message, TRUE);			
+	if (type == PURPLE_CONV_TYPE_CHAT)
+		fb_conversation_handle_chat(fba, from, to, message_time, message, TRUE);
+	else
+		fb_conversation_handle_message(fba, from, to, message_time, message, TRUE);			
 }
 
 static void parse_new_messages(PurpleConnection *pc, FacebookAccount *fba, JsonArray *messages)
@@ -119,7 +125,7 @@ static void parse_new_messages(PurpleConnection *pc, FacebookAccount *fba, JsonA
 
 		if (from && to && g_str_equal(type, "msg")) {
 			JsonObject *messageObj = json_node_get_object(json_object_get_member(object, "msg"));
-			parse_message(pc, fba, messageObj, from, to);
+			parse_message(pc, fba, messageObj, from, to, PURPLE_CONV_TYPE_IM);
 		} else if (from && g_str_equal(type, "typ")) {
 			purple_debug_info("facebook", "handling typing notification\n");
 
@@ -129,6 +135,36 @@ static void parse_new_messages(PurpleConnection *pc, FacebookAccount *fba, JsonA
 			} else {
 				serv_got_typing(pc, from, 10, PURPLE_TYPING);
 			}
+		} else if (g_str_equal(type, "group_msg")) {
+			PurpleConversation *conv = fb_find_chat(fba, to);
+			
+			if (conv != NULL)
+			{			
+				if (json_object_has_member(object, "to_name"))
+				{
+					const gchar *to_name = json_node_get_string(json_object_get_member(
+						object, "to_name"));
+					purple_conversation_set_title(conv, to_name);
+
+					PurpleChat *chat = purple_blist_find_chat(fba->account, to);
+					purple_blist_alias_chat(chat, to_name);
+				}
+				if (!purple_conv_chat_cb_find(PURPLE_CONV_CHAT(conv), from))
+				{
+					purple_conv_chat_add_user(PURPLE_CONV_CHAT(conv), from, NULL, PURPLE_CBFLAGS_NONE, FALSE);
+				}
+				if (!purple_find_buddy(fba->account, from))
+				{
+					const char *from_name = json_node_get_string(json_object_get_member(
+						object, "from_name"));
+					buddy = purple_buddy_new(fba->account, from, from_name);
+					purple_blist_node_set_flags((PurpleBlistNode *)buddy, PURPLE_BLIST_NODE_FLAG_NO_SAVE);
+					purple_blist_add_buddy(buddy, NULL, purple_find_group(DEFAULT_GROUP_NAME), NULL);
+				}
+			}
+
+			JsonObject *messageObj = json_node_get_object(json_object_get_member(object, "msg"));
+			parse_message(pc, fba, messageObj, from, to, PURPLE_CONV_TYPE_CHAT);
 		}
 
 		/*
@@ -242,6 +278,12 @@ static void got_new_messages(FacebookAccount *fba, const gchar *data,
 				json_node_get_array(json_object_get_member(objnode, "ms")));
 		}
 	}
+	
+	if (json_object_has_member(objnode, "s"))
+	{
+		gint new_seq = json_node_get_int(json_object_get_member(objnode, "s"));
+		fba->message_fetch_sequence = new_seq;
+	}
 
 	json_parser_free(parser);
 
@@ -362,7 +404,7 @@ static gboolean fb_send_im_fom(FacebookOutgoingMessage *msg)
 	g_free(encoded_message);
 	g_free(jstime);
 
-	fb_post_or_get(msg->fba, FB_METHOD_POST, NULL, "/ajax/chat/send.php", postdata, fb_send_im_cb, msg, FALSE);
+	fb_post_or_get(msg->fba, FB_METHOD_POST, NULL, "/ajax/chat/send.php?__a=1", postdata, fb_send_im_cb, msg, FALSE);
 	g_free(postdata);
 
 	return FALSE;
@@ -408,6 +450,7 @@ void got_reconnect_json(FacebookAccount *fba, const gchar *data, gsize data_len,
 {
 	JsonParser *parser;
 	JsonObject *objnode;
+	gchar *error_message;
 
 	parser = fb_get_parser(data, data_len);
 
@@ -420,7 +463,21 @@ void got_reconnect_json(FacebookAccount *fba, const gchar *data, gsize data_len,
 		return;
 	}
 
-	objnode = fb_get_json_object(parser, NULL);
+	objnode = fb_get_json_object(parser, &error_message);
+	
+	if (error_message != NULL)
+	{
+		if (json_node_get_int(json_object_get_member(objnode, "error")) == 1356007)
+		{
+			//There'll normally be an error message if chat is down for maintenance
+			purple_connection_error_reason(fba->pc,
+				PURPLE_CONNECTION_ERROR_AUTHENTICATION_IMPOSSIBLE,
+				error_message);
+			g_free(error_message);
+			json_parser_free(parser);
+			return;
+		}
+	}
 
 	JsonObject *payload = json_node_get_object(json_object_get_member(objnode, "payload"));
 	
@@ -454,7 +511,7 @@ void got_reconnect_json(FacebookAccount *fba, const gchar *data, gsize data_len,
 
 gboolean fb_reconnect(FacebookAccount *fba)
 {
-	gchar *url = g_strdup_printf("/ajax/presence/reconnect.php?reason=7&post_form_id=%s&__a=1", fba->post_form_id);
+	gchar *url = g_strdup_printf("/ajax/presence/reconnect.php?__a=1&reason=7&post_form_id=%s&__a=1", fba->post_form_id);
 	fb_post_or_get(fba, FB_METHOD_GET, NULL, url, NULL, got_reconnect_json, NULL, FALSE);
 	g_free(url);
 	
