@@ -215,7 +215,7 @@ JsonObject *fb_get_json_object(JsonParser *parser, char **error_message)
 		if (error_number)
 		{
 			purple_debug_error("facebook",
-				"got error from facebook of %s (%s)",
+				"got error from facebook of %s (%s)\n",
 				summary, description);
 			// Pass error message to calling function if they asked for it.
 			if (error_message) {
