@@ -28,6 +28,7 @@
 #include "fb_notifications.h"
 #include "fb_search.h"
 #include "fb_friendlist.h"
+#include "fb_chat.h"
 
 static void fb_login_cb(FacebookAccount *fba, const gchar *response, gsize len,
 		gpointer userdata);
@@ -79,18 +80,16 @@ static GList *fb_statuses(PurpleAccount *account)
 
 	/* Online people have a status message and also a date when it was set */
 	status = purple_status_type_new_with_attrs(PURPLE_STATUS_AVAILABLE,
-		NULL, _("Online"), FALSE, TRUE, FALSE, "message",
+		NULL, _("Online"), TRUE, TRUE, FALSE, "message",
 		_("Message"), purple_value_new(PURPLE_TYPE_STRING), NULL);
 	types = g_list_append(types, status);
 	
 	/* Cave into feature requests and allow people to set themselves to be idle */
-	status = purple_status_type_new_with_attrs(PURPLE_STATUS_AWAY,
-		NULL, _("Idle"), FALSE, TRUE, FALSE, "message",
-		_("Message"), purple_value_new(PURPLE_TYPE_STRING), NULL);
+	status = purple_status_type_new_full(PURPLE_STATUS_AWAY, NULL, _("Idle"), FALSE, TRUE, FALSE);
 	types = g_list_append(types, status);
 
 	/* Offline people dont have messages */
-	status = purple_status_type_new_full(PURPLE_STATUS_OFFLINE, NULL, _("Offline"), FALSE, TRUE, FALSE);
+	status = purple_status_type_new_full(PURPLE_STATUS_OFFLINE, NULL, _("Offline"), TRUE, TRUE, FALSE);
 	types = g_list_append(types, status);
 
 	return types;
@@ -128,7 +127,7 @@ static gboolean fb_get_messages_failsafe(FacebookAccount *fba)
 
 void fb_login_captcha_cancel_cb(PurpleConnection *pc, PurpleRequestFields *fields)
 {
-	purple_connection_error_reason(pc, PURPLE_CONNECTION_ERROR_OTHER_ERROR,
+	purple_connection_error_reason(pc, PURPLE_CONNECTION_ERROR_AUTHENTICATION_FAILED,
 		"Could not authenticate captcha.  Logging into the Facebook website may fix this.");
 }
 
@@ -305,6 +304,7 @@ static void fb_login_cb(FacebookAccount *fba, const gchar *response, gsize len,
 	/* Check for new 'device login' security setting */
 	if (g_hash_table_lookup(fba->cookie_table, "lgnntfy") && !g_hash_table_lookup(fba->cookie_table, "c_user"))
 	{
+		purple_debug_info("facebook", "Registering computer name\n");
 		/* Facebook may need us to identify ourselves */
 		fb_post_or_get(fba, FB_METHOD_POST | FB_METHOD_SSL, "login.facebook.com",
 			"/loginnotify/setup_machine.php", "machinename=libpurple&remembercomputer=1", fb_login_cb, NULL, FALSE);
@@ -331,6 +331,9 @@ static void fb_login_cb(FacebookAccount *fba, const gchar *response, gsize len,
 	/* ok, we're logged in now! */
 	purple_connection_set_state(fba->pc, PURPLE_CONNECTED);
 
+	/* Download a list of all group-chat's we belong to */
+	fb_get_groups(fba);
+
 	/* This will kick off our long-poll message retrieval loop */
 	fb_get_post_form_id(fba, &fb_post_form_id_cb);
 	fb_check_friend_requests(fba);
@@ -420,6 +423,9 @@ static void fb_login(PurpleAccount *account)
 {
 	FacebookAccount *fba;
 	
+	/* Force disable captcha */
+	purple_account_set_bool(account, "ignore-facebook-captcha", TRUE);
+	
 	/* Create account and initialize state */
 	fba = g_new0(FacebookAccount, 1);
 	fba->account = account;
@@ -542,7 +548,7 @@ static unsigned int fb_send_typing(PurpleConnection *pc, const gchar *name,
 		postdata = g_strdup_printf("typ=%d&to=%s&post_form_id=%s",
 				typing_state, encoded_name, fba->post_form_id);
 		g_free(encoded_name);
-		fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/chat/typ.php",
+		fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/chat/typ.php?__a=1",
 				postdata, NULL, NULL, FALSE);
 		g_free(postdata);
 	} else {
@@ -579,15 +585,15 @@ static void fb_set_status_ok_cb(gpointer data, const gchar *status_text)
 	if (*status_text_new != '\0')
 	{
 		status_tmp = g_strdup(purple_url_encode(status_text_new));
-		postdata = g_strdup_printf("profile_id=%" G_GINT64_FORMAT "&status=%s&post_form_id=%s",
-				fba->uid, status_tmp, fba->post_form_id);
+		postdata = g_strdup_printf("profile_id=%" G_GINT64_FORMAT "&status=%s&post_form_id=%s&fb_dtsg=%s",
+				fba->uid, status_tmp, fba->post_form_id, fba->dtsg);
 		g_free(status_tmp);
 	}
 	else
-		postdata = g_strdup_printf("profile_id=%" G_GINT64_FORMAT "&clear=1&post_form_id=%s",
-				fba->uid, fba->post_form_id);
+		postdata = g_strdup_printf("profile_id=%" G_GINT64_FORMAT "&clear=1&post_form_id=%s&fb_dtsg=%s",
+				fba->uid, fba->post_form_id, fba->dtsg);
 
-	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/updatestatus.php",
+	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/updatestatus.php?__a=1",
 			postdata, NULL, NULL, FALSE);
 
 	g_free(postdata);
@@ -822,8 +828,8 @@ static PurplePluginProtocolInfo prpl_info = {
 	fb_tooltip_text,        /* tooltip_text */
 	fb_statuses,            /* status_types */
 	fb_node_menu,           /* blist_node_menu */
-	NULL,                   /* chat_info */
-	NULL,                   /* chat_info_defaults */
+	fb_chat_info,           /* chat_info */
+	fb_chat_info_defaults,  /* chat_info_defaults */
 	fb_login,               /* login */
 	fb_close,               /* close */
 	fb_send_im,             /* send_im */
@@ -842,13 +848,13 @@ static PurplePluginProtocolInfo prpl_info = {
 	NULL,                   /* rem_permit */
 	NULL,                   /* rem_deny */
 	NULL,                   /* set_permit_deny */
-	NULL,                   /* join_chat */
+	fb_fake_join_chat,      /* join_chat */
 	NULL,                   /* reject chat invite */
-	NULL,                   /* get_chat_name */
+	fb_get_chat_name,       /* get_chat_name */
 	NULL,                   /* chat_invite */
-	NULL,                   /* chat_leave */
+	fb_chat_fake_leave,     /* chat_leave */
 	NULL,                   /* chat_whisper */
-	NULL,                   /* chat_send */
+	fb_chat_send,           /* chat_send */
 	NULL,                   /* keepalive */
 	NULL,                   /* register_user */
 	NULL,                   /* get_cb_info */
